<?php
/*
Plugin Name: Enhanced Search Form
Plugin URI: http://oogami.name/
Description: Enhance wordpress search form to allow searching posts in certain category, month archive or tag.
Author: Ono Oogami, BestWebLayout
Version: 0.5
Author URI: http://oogami.name/
License: GPLv3 or later
*/

/*
Copyright (C) 2009 Ono Oogami, http://oogami.name/ (ono@oogami.name)

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program. If not, see <http://www.gnu.org/licenses/>.
*/

/*
* Function to add localization to the plugin.
*/
if ( ! function_exists ( 'esf_init' ) ) {
	function esf_init() {
		load_plugin_textdomain( 'esf', false, dirname( plugin_basename( __FILE__ ) ) . '/languages/' );
	}
}

/*
* By default if you pass cat[] / tag[] as a Array GET/POST variables to server, convert cat[] / tag[] to string.
*/
if ( ! function_exists( 'esf_search_filter' ) ) {
	function esf_search_filter() {
		if ( is_search() && ( isset( $_GET['cat'] ) && is_array( $_GET['cat'] ) ) || ( isset( $_GET['tag'] ) && is_array( $_GET['tag'] ) ) ) {

			$esf_search = '?s=' . str_replace(' ', '+', $_GET['s'] );

			if ( $_GET['cat'] ) {
				$esf_cat_string = implode( ',', $_GET['cat'] );
				$esf_cat = ! empty( $esf_cat_string ) ? '&cat=' . urlencode( $esf_cat_string ) : '';
				$vars['cat'] = $esf_cat_string;
			}

			if ( isset( $_GET['m'] ) ) {
				$esf_month = ! empty( $_GET['m'] ) ? '&m=' . $_GET['m'] : '';
			}

			if ( $_GET['tag'] ) {
				$esf_tag_string = implode( ',', $_GET['tag'] );
				$esf_tag = ! empty( $esf_tag_string ) ? '&tag=' . urlencode( $esf_tag_string ) : '';
				$vars['tag'] = $esf_tag_string;
			}

			wp_redirect( get_site_url() . '/' . $esf_search . $esf_cat . $esf_month . $esf_tag );
			exit;
		}
	}
}

/*
* Function for insert advanced options in the search form.
*/
if ( ! function_exists( 'esf_search_form' ) ) {
	function esf_search_form( $form ) {
		return preg_replace( '/((<\/div\s*>\s*)?<\/form\s*>)/', esf_add_form_field( true ) . "\\1", $form, 1 );
	}
}

/*
* Function to create advanced options.
*/
if ( ! function_exists( 'esf_add_form_field' ) ) {
	function esf_add_form_field( $return = false ) {
		global $wpdb, $wp_locale;

		$esf_get_categories = get_categories(
			array(
				'hierarchical' => 0
			)
		);

		if ( $esf_get_categories ) {
			$esf_in_cats = isset( $_GET['cat'] ) ? explode( ',', $_GET['cat'] ) : '';
			$esf_categories_id = uniqid( 'esf_cat_' );
			$esf_categories = sprintf( '<p class="esf_wrapper"><label for="%s">%s</label></p>', $esf_categories_id, __( 'Search by categories:', 'esf' ) );
			$esf_categories .= sprintf( '<select id="%s" class="esf_cat" name="cat[]" multiple="multiple" size="5">', $esf_categories_id );
			$esf_categories .= sprintf( '<option value="" %s>%s</option>', get_query_var( 'cat' ) ? '' : 'selected="selected"', __( 'ALL', 'esf' ) );
			foreach( $esf_get_categories as $esf_cat ){
				$esf_categories .= sprintf( '<option value="%s" %s>%s</option>', $esf_cat->term_id, ( isset( $_GET['cat'] ) && in_array( $esf_cat->term_id, $esf_in_cats ) ) ? 'selected="selected"' : '', $esf_cat->name );
			}
			$esf_categories .= '</select>';
		}

		$esf_query = "SELECT YEAR(post_date) AS `year`, MONTH(post_date) AS `month`, count(ID) as posts FROM $wpdb->posts WHERE post_type = 'post' AND post_status = 'publish' GROUP BY YEAR(post_date), MONTH(post_date) ORDER BY post_date DESC";
		$esf_key = md5( $esf_query );
		$esf_cache = wp_cache_get( 'wp_get_archives', 'general' );

		if ( ! isset( $esf_cache[ $esf_key ] ) ) {
			$esf_results = $wpdb->get_results( $esf_query );
			$esf_cache[ $esf_key ] = $esf_results;
			wp_cache_add( 'wp_get_archives', $esf_cache, 'general' );
		} else {
			$esf_results = $esf_cache[ $esf_key ];
		}

		if ( $esf_results ) {
			$esf_archives_id = uniqid( 'esf_arch_' );
			$esf_archives = sprintf( '<p class="esf_wrapper"><label for="%s">%s</label></p>', $esf_archives_id, __( 'Search by month:', 'esf' ) );
			$esf_archives .= sprintf( '<select id="%s" class="esf_arch" name="m">', $esf_archives_id );
			$esf_archives .= sprintf( '<option value="" %s>%s</option>', get_query_var( 'm' ) ? '' : 'selected="selected"', __( 'ALL', 'esf' ) );
			foreach( (array) $esf_results as $esf_result ) {
				if ( $esf_result->year == 0 || $esf_result->month == 0 ) {
					continue;
				}
				$esf_archives .= sprintf( '<option value="%s" %s>%s %s</option>', $esf_result->year . zeroise( $esf_result->month, 2 ), ( get_query_var( 'm' ) == $esf_result->year . zeroise( $esf_result->month, 2 ) ) ? 'selected="selected"' : '', $wp_locale->get_month( $esf_result->month ), $esf_result->year );
			}
			$esf_archives .= '</select>';
		}

		$esf_get_tags = get_tags(
			array(
				'number' => 45,
				'orderby' => 'count',
				'order' => 'DESC'
			)
		);

		if ( $esf_get_tags ) {
			$esf_in_tags = isset( $_GET['tag'] ) ? explode( ',', $_GET['tag'] ) : '';
			$esf_tags_id = uniqid( 'esf_tag_' );
			$esf_tags = sprintf( '<p class="esf_wrapper"><label for="%s">%s</label></p>', $esf_tags_id, __( 'Search by tags:', 'esf' ) );
			$esf_tags .= sprintf( '<select id="%s" class="esf_tag" name="tag[]" multiple="multiple" size="5">', $esf_tags_id );
			$esf_tags .= sprintf( '<option value="" %s>%s</option>', get_query_var( 'tag' ) ? '' : 'selected="selected"', __( 'ALL', 'esf' ) );
			foreach ( $esf_get_tags as $esf_tag ) {
				$esf_tags .= sprintf( '<option value="%s" %s>%s</option>', $esf_tag->slug, ( isset( $_GET['tag'] ) && in_array( $esf_tag->slug, $esf_in_tags ) ) ? 'selected="selected"' : '', $esf_tag->name );
			}
			$esf_tags .= '</select>';
		}

		$esf_search_options = sprintf( '<input id="esf_advanced" class="esf_advanced" type="button" value="%s" /><div class="esf_search_options" style="display:%s;">%s</div>', __( 'Advanced', 'esf' ), is_search() ? 'block' : 'none', $esf_categories . $esf_archives . $esf_tags );

		if ( $return ) {
			return $esf_search_options;
		}

		echo $esf_search_options;
	}
}

if ( ! function_exists( 'esf_frontend_head' ) ) {
	function esf_frontend_head() {
		wp_enqueue_style( 'esf_style', plugins_url( 'css/style.css', __FILE__ ) );
		wp_enqueue_script( 'esf_script', plugins_url( 'js/script.js', __FILE__ ), array( 'jquery' ) );
	}
}

/* Initialization */
add_action( 'init', 'esf_init' );
/* Fix GET/POST variables */
add_filter( 'pre_get_posts', 'esf_search_filter' );
/* Insert advanced options in the search form */
add_filter( 'get_search_form', 'esf_search_form' );
/* Adding scripts and styles to the frontend */
add_action( 'wp_enqueue_scripts', 'esf_frontend_head' );
?>